<?php
//
// $Id: tina_mvc.php, v 0.00 Sat Jan 30 2010 15:42:33 GMT+0000 (IST) Francis Crossen $
//

/*
Plugin Name: Tina MVC
Plugin URI: http://seeit.org/tina-mvc-for-wordpress/
Description: The Tina MVC framework allows you to develop plugins using a MVC design pattern. Tina controllers are accessed through any of: 1) a Wordpress page (which acts as a font-end controller to Tina); 2) a controller call from a Tina MVC Wordpress Widget; and 3) a controller call via a Wordpress shortcode call. Developed on PHP version 5.2 and Wordpress 2.9.1+.
Author: Francis Crossen <francis@crossen.org>
Version: 0.1.3
Author URI: http://SeeIT.org/
License: GPL2
*/
/*  Copyright 2010 Francis Crossen (email: francis@crossen.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * Tina MVC (Model-View-Controller) Wordpress Plugin - main plugin file.
 *
 * The Tina MVC framework allows you to develop plugins using a MVC design pattern.
 * Tina controllers are accessed through any of:
 *  - a Wordpress page (which acts as a font-end controller to Tina)
 *  - a controller call from a Tina MVC Wordpress Widget
 *  - a controller call via a Wordpress shortcode call
 *
 * PHP version 5 and Wordpress 2.9.1.
 *
 * LICENSE: This source file is currently licensed for use by the author only.
 * For all enquiries contact the author.
 *
 * @package    Tina-MVC
 * @subpackage    Core
 * @author     Francis Crossen <francis@crossen.org>
 * @copyright  Francis Crossen <francis@crossen.org>
 * @license    GPL2
 */

/**
 * Include various Tina Framework utility functions.
 *
 * Only if our plugin has been activated
 */
if( get_option('tina_mvc_plugin_active') ) {
  require(WP_PLUGIN_DIR.'/tina-mvc/tina_mvc/tina_mvc_functions.php');   
}

/**
 * Register the activation hook
 */
register_activation_hook(__FILE__,'tina_mvc_install'); 

/**
 * Wordpress function that installs the plugin - runs when plugin is activated
 *
 * Install the Wordpress plugin. Takes settings from 'tina_mvc_app_settings_sample.php' and
 * 'tina_mvc_app_settings.php' and creates Wordpress pages accordingly.
 * 
 * Those pages become front end controllers for the Tina framework.
 *
 * @see  tina_mvc_app_settings_sample.php
 * @todo  Allow for custom DB tables
 */
function tina_mvc_install() {
    
    global $wpdb;
    
    // find an app settings file...
    if( file_exists( WP_PLUGIN_DIR.'/tina-mvc/tina_mvc_app_settings.php' ) ) {
      include( WP_PLUGIN_DIR.'/tina-mvc/tina_mvc_app_settings.php' );
    }
    else {
      include( WP_PLUGIN_DIR.'/tina-mvc/tina_mvc_app_settings_sample.php' );
    }
    
    // for the Wordpress options
    $tina_mvc_options_pages = array();
    
    // grab the default page text.
    include( WP_PLUGIN_DIR.'/tina-mvc/tina_mvc_inc/tina_mvc_default_page_post.php' );
    $post_content = get_tina_mvc_default_page_post_content();
    
    // loop through the pages....
    foreach( $tina_mvc_pages AS $p ) {
      
      // does the page exist...?
      $the_page = get_page_by_title( $p['page_title'] );
      
      if ( ! $the_page ) {
        
        // Create post object
        $_p = array();
        $_p['post_title'] = $p['page_title'];
        $_p['post_name'] = $p['page_name'];
        $_p['post_content'] = $post_content;
        $_p['post_status'] = $p['page_status'];
        $_p['post_type'] = 'page';
        $_p['comment_status'] = 'closed';
        $_p['ping_status'] = 'closed';
        $_p['post_category'] = array(1);
        
        // Insert the post into the database
        $the_page_id = wp_insert_post( $_p );
        
      }
      else {
        
        $the_page_id = $the_page->ID;
        
        //make sure the page is not trashed...
        $the_page->post_status = $p['page_status'];
        $the_page_id = wp_update_post( $the_page );
        
      }
      
      // get the page in case Wordpress messed with the page_name
      $_p = get_page( $the_page_id );
      
      // FOR the options
      $page_name = $_p->post_name;
      $page_id = $_p->ID;
      $page_data = array( 'page_title'=>$_p->post_title, 'page_name'=>$page_name, 'page_id'=>$page_id ); // handy to have them for later...
      
      $tina_mvc_options_pages[$page_name] = $page_data;
      $tina_mvc_options_pages[$page_id] = $page_data;
      
    }
    
    delete_option("tina_mvc_pages");
    add_option("tina_mvc_pages", $tina_mvc_options_pages, '', 'yes');
    
    // the default role to view...
    $_opt = $tina_mvc_default_role_to_view;    
    delete_option("tina_mvc_default_role_to_view");
    add_option("tina_mvc_default_role_to_view", $_opt, '', 'yes');
    
    // the tina_mvc_login_logout_redirect...
    $_opt = $tina_mvc_login_logout_redirect;
    delete_option("tina_mvc_login_logout_redirect");
    add_option("tina_mvc_login_logout_redirect", $_opt, '', 'yes');
    
    // the default capability to view...
    $_opt = $tina_mvc_default_capability_to_view;
    delete_option("tina_mvc_default_capability_to_view");
    add_option("tina_mvc_default_capability_to_view", $_opt, '', 'yes');
    
    // the reCaptcha api keys
    $_opt = $tina_mvc_recaptcha_pub_key;
    delete_option("tina_mvc_recaptcha_pub_key");
    add_option("tina_mvc_recaptcha_pub_key", $_opt, '', 'yes');
    $_opt = $tina_mvc_recaptcha_pri_key;
    delete_option("tina_mvc_recaptcha_pri_key");
    add_option("tina_mvc_recaptcha_pri_key", $_opt, '', 'yes');
    
    // need to add roles...? Why not use the Roles and Capabilities plugin?
    // $RoleObj = new WP_Roles();
    // $RoleObj->add_role( 'role-nickname', 'Role Long Name', $capabilities = array('do_whatever') );    
    
    // finally
    add_option("tina_mvc_plugin_active", TRUE, '', 'yes');
    
}

/**
 * Register the deactivation hook
 */
register_deactivation_hook( __FILE__, 'tina_mvc_remove' );
/**
 * Wordpress function that removes the plugin - runs on deactivation
 */
function tina_mvc_remove() {
    
    global $wpdb;
    
    //  the id of our pages...
    $the_pages = ( get_option( 'tina_mvc_pages' ) );
    
    if( $the_pages ) {
      
      foreach( $the_pages AS $_p ) {
        
        wp_delete_post( $_p['ID'] );
        
      }
      
    }
    
    delete_option("tina_mvc_pages");
    delete_option("tina_mvc_default_role_to_view");
    delete_option("tina_mvc_login_logout_redirect");
    delete_option("tina_mvc_default_capability_to_view");
    delete_option("tina_mvc_recaptcha_pub_key");
    delete_option("tina_mvc_recaptcha_pri_key");
    // finally
    delete_option('tina_mvc_plugin_active');
    
}

/**
 * Administration actions
 */
if ( is_admin() ){

    function tina_mvc_admin_html_page() {
        add_options_page('Tina MVC Options', 'Tina MVC', 'administrator', 'tina_mvc_options', 'tina_mvc_admin_settings_options_html');
    }
    add_action('admin_menu', 'tina_mvc_admin_html_page');
    
    function tina_mvc_admin_settings_options_html() {
        require('tina_mvc_inc/tina_mvc_admin_html_page.php');
    }

}

/**
 * Flag a Tina MVC front end controller (page) request and not a widget or shortcode request.
 * This is used instead of a constant as we may have multiple Tina MVC controller calls from a page
 * (for example, page, widget and shortcode) and need to be able to change its value
 */
$tina_mvc_page_request = false;

/**
 * Tina MVC autoload function - only defined for Tina MVC requests
 *
 * Page controllers are located in the `tina_mvc/` or the `app/` folder. If a Tina controller
 * is called from a front end page controller, then the page controllers can be in a subfolder
 * with the same name as the 'post_name' (slug) as the front end controller page. By default these are
 * called `tina` (a private page) and `tina-mvc-for-wordpress` (a public page).
 *
 * You can also place your controllers in the app folder. If you do they will be
 * available through all of your front end page controllers.
 *
 * @param   string $class_name
 */
function __autoload( $class_name) {
    
  // TINA_MVC_CALL is defined if we have a Tina call active
  if( defined('TINA_MVC_CALL') ) {
    
    $plugin_path = WP_PLUGIN_DIR.'/tina-mvc/';
    $my_page_controller = false;    
    $common_page_controller = false;
    $tina_mvc_page_request = false;
    
    global $tina_controller_called_from_page_filter;
    
    // are we called from a front end controller page?
    if( defined('TINA_MVC_PAGE_CONTROLLER_NAME') ) {
      $my_page_controller = $plugin_path.'app/'.TINA_MVC_PAGE_CONTROLLER_NAME."/$class_name.php";
      $tina_mvc_page_request = true;
    }
    elseif( defined('TINA_MVC_PAGE_CONTROLLER_ID') ) {
      // find the page name...
      $tina_pages = get_option('tina_mvc_pages');
      $my_page_controller = $plugin_path.'app/'.$tina_pages[TINA_MVC_PAGE_CONTROLLER_ID]['page_name']."/$class_name.php";
      $tina_mvc_page_request = true;
    }
    
    // look in the app folder for any common controllers. your shortcode and widget controllers go here too
    $common_page_controller = $plugin_path."app/$class_name.php";
    
    if( $my_page_controller AND file_exists( $my_page_controller ) ) {
      include( $my_page_controller );
    }
    elseif( file_exists($common_page_controller) ) {
      include( $common_page_controller );
    }
    elseif ( file_exists(  $plugin_path.'tina_mvc/'.$class_name.'.php' ) ) {
      include( $plugin_path.'tina_mvc/'.$class_name.'.php' );
    }
    elseif( !empty($tina_controller_called_from_page_filter) ) {
      
      // set what we need to display a 404...
      global $wp_query;
      $wp_query->is_404 = true;
      $wp_query->is_single = false;
      $wp_query->is_page = false;
      
      //echo "<pre><small>".print_r($wp_query,1); die();
      
      include( get_query_template( '404' ) );
      exit();
      
    }
    else {
      
      echo 'tina_mvc.php::' . __FUNCTION__ . ' (' . __LINE__ . ') Class \'' . $class_name . '\' not found. Looked for:';
      if( $my_page_controller ) {
        echo "<br /> - $my_page_controller";
      }
      echo "<br /> - $common_page_controller";
      echo "<br /> - $plugin_path".'tina_mvc/'.$class_name.'.php';
      exit();
      
    }
    
  }
  
}

/**
 * Page filter to detect a call to our controller and to pass control to it
 *
 * Checks $wp_query->get('tina_mvc_request') and if set triggers a call to
 * the Tina MVC framework. Sets the Wordpress post_title and post_content.
 * The (object) $wp_query is previously marked tina_mvc_query_parser()
 * to flag a call to our controller
 *
 * There is a Wordpress quirk that looses the type of the 'tina_mvc_default_role_to_view'
 * option. In the tine_mvc_settings.php file we use -1 instead of a boolean false
 * 
 * @param   array $posts A single element array of posts. This can be empty for pages like 'tina/some-page'
 * @return  array $posts
 * @todo    Proper 404 error handling
 */
function tina_mvc_page_filter( $posts ) {

    global $wp_query;

    if( $tina_mvc_request = $wp_query->get('tina_mvc_request') ) {
      
      //  if we have not been passed $posts, then generate an empty one...
      if( ! $posts ) {
        $posts = array( tina_mvc_get_empty_page_post() ); 
      }
        
        // echo "<pre>posts: ".print_r( $posts, 1 ).  print_r($tina_mvc_request,1). "</pre>"; // $posts is empty if it is a controller/subcontroller/whatever call

        define('TINA_MVC_CALL', true); // so we can sniff it in our __autoload() function
        
        // Required so we can make links using the page controller, remember we can have multiple front end page controllers.
        // This also serves to flag our tina_mvc_controlelr_class() that it has been called from this page filter
        // define('TINA_MVC_PAGE_CONTROLLER_ID', $posts[0]->ID ); 
        
        // we use a global variable here because we may have multiple Tina controller calls in one page request and cannot redefine a constant...
        global $tina_mvc_page_request; // flags a page request, NOT a widget or shortcode request
        $tina_mvc_page_request = true;
        
        // we need to force a boolean value here...
        if( ($default_role_to_view = get_option('tina_mvc_default_role_to_view')) == -1 ) {
            $default_role_to_view = false;
        }
        
        $default_capability_to_view = get_option('tina_mvc_default_capability_to_view');
        
        // echo "<pre>\$default_role_to_view: ".var_dump( $default_role_to_view ).' // '.var_dump( $default_capability_to_view )."</pre>";
        // echo "<pre>app: ".print_r( $tina_mvc_request, 1 )."</pre>";
        
        $APP = new tina_mvc_controller_class( $tina_mvc_request, $default_role_to_view, $default_capability_to_view, $called_from=FALSE );
        // if we have no app, we unset the posts and allow WP to use whatever 404 page it can find...
        
        if( $APP ) {
            // we only have one post at $posts[0]
            if( $APP->get_post_title() ) {
              $posts[0]->post_title = $APP->get_post_title();            
            }
            $posts[0]->post_content = $APP->get_post_content();
            
        }
        else {
            $posts = array();
            $wp_query->post_count = 0;
        }
        
    }
    
    return $posts;
  
}
add_filter( 'the_posts', 'tina_mvc_page_filter' );

/**
 * Handler to parse the wordpress WP_Query object.
 *
 * Looks at the Wordpress query object and checks to see if a page has been
 * called and if that pages matches our front end controller.
 * If there is a match we mark the WP_Query object so tina_mvc_page_filter()
 * function is triggered.
 *
 * @param   object $q  WP_Query object passed by Wordpress
 * @return  object WP_Query object
 */
function tina_mvc_query_parser( $q ) {

    $tina_mvc_pages = get_option( "tina_mvc_pages" );
    
    if( isset( $q->query_vars['page_id'] ) AND ( array_key_exists( $q->query_vars['page_id'], $tina_mvc_pages ) ) ) {
      
      define('TINA_MVC_PAGE_CONTROLLER_ID', $q->query_vars['page_id'] );
      
      // remove the page id from the query string...
      $tina_mvc_request = str_ireplace( 'page_id='.$q->query_vars['page_id'] , '', $_SERVER['QUERY_STRING'] );
      // is there a trailing slash?
      
      if( $tina_mvc_request[0] == '/' ) {
        $tina_mvc_request = substr($tina_mvc_request,1);
      }
      
      if( ! $tina_mvc_request ) {
        $tina_mvc_request = 'index';
      }
      
      $q->set('tina_mvc_request', $tina_mvc_request );
      $q->set('page_id', $q->query_vars['page_id'] );
      
    }
    elseif( isset( $q->query_vars['pagename'] ) 
          AND ( ( array_key_exists( $q->query_vars['pagename'], $tina_mvc_pages ) ) OR ( ($_sl_pos=strpos( $q->query_vars['pagename'], '/' ) ) AND ( array_key_exists( substr($q->query_vars['pagename'], 0, $_sl_pos ), $tina_mvc_pages ) ) ) ) ) {
      
      // no need for a boolean false check above - the '/' will never be at position 0
      // USING PERMALINKS
      
      if( strpos( $q->query_vars['pagename'], '/' ) === false ) { // we are just calling the base controller
        
        $tina_mvc_request = 'index';
        define('TINA_MVC_PAGE_CONTROLLER_NAME', $q->query_vars['pagename'] );
        
      }
      else {
        
        // we need to get the action from the query strings... e.g. /myplugin/user/edit/3
        /*
        for example in each case below we want 'myplugin/test-form/1/2/3'
          
            [REQUEST_URI] => /francis/tina/myplugin/test-form/1/2/3
            [SCRIPT_NAME] => /francis/tina/index.php
            [PHP_SELF] => /francis/tina/index.php
            
            [REQUEST_URI] => /myplugin/test-form/1/2/3
            [SCRIPT_NAME] => /index.php
            [PHP_SELF] => /index.php
        */
        
        $_slash_pos = strrpos( $_SERVER['PHP_SELF'], '/' );
        $_bit_we_dont_want = substr( $_SERVER['REQUEST_URI'], 0, $_slash_pos+1 );
        $tina_mvc_request = substr( $_SERVER['REQUEST_URI'], strlen($_bit_we_dont_want) );
        
        // now extract the tina front end controller page from the request - required so we can construct links to it
        define('TINA_MVC_PAGE_CONTROLLER_NAME', substr( $tina_mvc_request, 0, strpos( $tina_mvc_request, '/' ) ) );
        $tina_mvc_request = substr( $tina_mvc_request, -1*(strlen($tina_mvc_request)-strlen(TINA_MVC_PAGE_CONTROLLER_NAME)-1) );
        
        // echo "PAGE= ".TINA_MVC_PAGE_CONTROLLER_NAME." // EXTENDED REQUEST = $tina_mvc_request <hr>";
        // echo print_r($_SERVER['PHP_SELF'],1) . '|' .  print_r($_SERVER['REQUEST_URI'],1) . '<hr>';            
        // echo print_r(  $tina_mvc_request  ,1) . '|'  . '<hr>';            
      
      }
      
      $q->set('tina_mvc_request', $tina_mvc_request );
      $q->set('tina_mvc_front_end_page_controller', $front_end_page_controller );
      
    }
    else {
      
      // NOT a Tina request. return here to allow easier debugging of Tina requests
      $q->set( 'tina_mvc_request', false );
      return $q;
    }
    
    return $q;

}
add_filter( 'parse_query', 'tina_mvc_query_parser' );

/**
 * Handler to parse Tina MVC shortcodes
 *
 * Shortcodes may be self-closing or enclosing.
 * 
 * @param   array $attributes the shortcode attributes. $attributes['controller'] is required
 * @return  string The pre-escaped HTML to display
 */
function tina_mvc_shortcode_func($attributes, $content=NULL) {

    $atts = shortcode_atts( array('controller' => 'index', 'role_to_view' => FALSE, 'capability_to_view' => FALSE), $attributes);
    
    define('TINA_MVC_CALL', true); // so we can sniff it in our __autoload() function
    
    global $tina_controller_called_from_page_filter;
    $tina_controller_called_from_page_filter = FALSE; // this is set so we can sniff it in our __autoload() function...
    
    // check for -1 being passed to us by the shortcode call
    if( $atts['role_to_view'] == '-1' ) {
      $atts['role_to_view'] = FALSE;
    }
    
    $APP = new tina_mvc_controller_class( $atts['controller'], $atts['role_to_view'], $atts['capability_to_view'], $called_from='SHORTCODE' );

    $ret = $APP->get_post_content();

    if( !is_null($content) AND !$ret ) {
        return $content;
    }
    else {
        return $ret;        
    }

}
add_shortcode('tina_mvc', 'tina_mvc_shortcode_func'); 

/**
 * Add function to widgets_init that'll load our widget.
 */
add_action( 'widgets_init', 'tina_mvc_load_widgets' );

/**
 * Register our widget.
 */
function tina_mvc_load_widgets() {
	register_widget( 'Tina_MVC_Widget' );
}

/**
 * The Tina MVC Widget class.
 * 
 * This class handles everything that needs to be handled with the widget
 * 
 * @package    Tina-MVC
* @subpackage    Core*/
class Tina_MVC_Widget extends WP_Widget {

    /**
     * Widget setup.
     */
    function Tina_MVC_Widget() {
      
      // settings
      $widget_ops = array( 'classname' => 'tina_mvc', 'description' => __('Call a Tina MVC framework controller', 'tina_mvc') );
      
      // Widget control settings
      $control_ops = array( 'width' => 250, 'height' => 300, 'id_base' => 'tina_mvc_widget' );
      
      // Create the widget
      $this->WP_Widget( 'tina_mvc_widget', __('Tina MVC Widget', 'tina_mvc'), $widget_ops, $control_ops );
      
    }

    /**
     * How to display the widget on the screen.
     */
    function widget( $args, $instance ) {
      
      extract( $args );
      
      // Our variables from the widget settings
      $title = apply_filters('widget_title', $instance['title'] );
      $controller = $instance['controller'];
      if( $instance['no_role_to_view'] ) {
        $role_to_view = FALSE;
      }
      elseif( $instance['role_to_view'] ) {
        $role_to_view = $instance['role_to_view'];
      }
      elseif( $instance['capability_to_view'] ) {
        $capability_to_view = $instance['capability_to_view'];
      }
      
      define('TINA_MVC_CALL', true); // so we can sniff it in our __autoload() function

      global $tina_controller_called_from_page_filter;
      $tina_controller_called_from_page_filter = FALSE; // this is set so we can sniff it in our __autoload() function...
        
      $APP = new tina_mvc_controller_class( $controller, $role_to_view, $capability_to_view, $called_from='WIDGET' );
      $content = $APP->get_post_content();
      
      if( $content ) {
          
        // Tina widget is only displayed if the permissions check passes
        
        // Before widget (defined by themes)
        echo $before_widget;
        
        // Display the widget title if one was input (before and after defined by themes)
        if( ! ( $tina_mvc_title = $APP->get_post_title() ) ) {
          $tina_mvc_title = $instance['title'];
        }
        echo $before_title . $tina_mvc_title . $after_title;
        
        echo $content;
        
        // After widget (defined by themes)
        echo $after_widget;
        
      }
      
    }

    /**
     * Update the widget settings.
     */
    function update( $new_instance, $old_instance ) {
      
      $instance = $old_instance;
      
      // Strip tags for title and name to remove HTML (important for text inputs)
      $instance['title'] = strip_tags( $new_instance['title'] );
      $instance['controller'] = strip_tags( $new_instance['controller'] );
      $instance['no_role_to_view'] = (  $new_instance['no_role_to_view']  ?  1  :  0  );
      $instance['role_to_view'] = strip_tags( $new_instance['role_to_view'] );
      $instance['capability_to_view'] = strip_tags( $new_instance['capability_to_view'] );
      
      return $instance;
    
    }

    /**
     * Displays the widget settings controls on the widget panel.
     * Make use of the get_field_id() and get_field_name() function
     * when creating your form elements. This handles the confusing stuff.
     */
    function form( $instance ) {

            /* Set up some default widget settings. */
            $defaults = array( 'title' => __('The default widget title', 'tina_mvc'), 'controller' => __('tina-mvc', 'tina_mvc') );
            $instance = wp_parse_args( (array) $instance, $defaults ); ?>

            <!-- Widget Title: Text Input -->
            <p>
                    <label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e('Default Widget Title:', 'hybrid'); ?></label>
                    <input id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo $instance['title']; ?>" style="width:100%;" />
            </p>

            <!-- Role To View: Checkbox -->
            <p>
                    <label for="<?php echo $this->get_field_id( 'no_role_to_view' ); ?>"><?php _e('Visible to all users (overrides all settings below)?', 'tina_mvc'); ?></label>
                    <input id="<?php echo $this->get_field_id( 'no_role_to_view' ); ?>" name="<?php echo $this->get_field_name( 'no_role_to_view' ); ?>" type="checkbox" value="1" <?php if( $instance['no_role_to_view'] ) echo 'checked'; ?> />
            </p>

            <!-- Controller: Text Input -->
            <p>
                    <label for="<?php echo $this->get_field_id( 'controller' ); ?>"><?php _e('Controller:', 'tina_mvc'); ?></label>
                    <input id="<?php echo $this->get_field_id( 'controller' ); ?>" name="<?php echo $this->get_field_name( 'controller' ); ?>" value="<?php echo $instance['controller']; ?>" style="width:100%;" />
            </p>

            <!-- Role To View: Text Input -->
            <p>
                    <label for="<?php echo $this->get_field_id( 'role_to_view' ); ?>"><?php _e('Role To View (empty means user must be logged in):', 'tina_mvc'); ?></label>
                    <input id="<?php echo $this->get_field_id( 'role_to_view' ); ?>" name="<?php echo $this->get_field_name( 'role_to_view' ); ?>" value="<?php echo $instance['role_to_view']; ?>" style="width:100%;" />
            </p>

            <!-- Capability To View: Text Input -->
            <p>
                    <label for="<?php echo $this->get_field_id( 'capability_to_view' ); ?>"><?php _e('Capability To View (overrides the above settings):', 'tina_mvc'); ?></label>
                    <input id="<?php echo $this->get_field_id( 'capability_to_view' ); ?>" name="<?php echo $this->get_field_name( 'capability_to_view' ); ?>" value="<?php echo $instance['capability_to_view']; ?>" style="width:100%;" />
            </p>

    <?php
    }
}

/**
 * Only enable the following actions if Tina MVC is active
 */
if( get_option('tina_mvc_plugin_active') AND (get_option('tina_mvc_default_role_to_view') != -1) ) {

  /**
   * Redirect-on-login to prevent 'Subscribers' from ever SEEING A Wordpress backend
   */
  function tina_mvc_logon_redirect( $user_login='' ) {
    
    if( ( get_option('tina_mvc_default_role_to_view') != -1 )
          AND !tina_mvc_user_has_role( explode(',',get_option('tina_mvc_login_logout_redirect')) ) ) {
      
      wp_redirect( tina_mvc_make_controller_url() );
      
    }
    
  }
  add_action( 'wp_login', 'tina_mvc_logon_redirect' );
  
  /**
   * Redirect-on-logout to prevent 'Subscribers' from ever SEEING A Wordpress backend
   */
  function tina_mvc_logout_redirect() {
    
      wp_redirect( tina_mvc_make_controller_url() );
      exit();
  }
  add_action( 'wp_logout', 'tina_mvc_logout_redirect' );
  
  /**
   * Prevent users from accessing /wp-admin unless they have appropriate role
   */
  function restrict_admin() {
      
      if( !tina_mvc_user_has_role( explode(',',get_option('tina_mvc_login_logout_redirect')) ) ) {
          
          wp_redirect( tina_mvc_make_controller_url() );
          exit();
          
      }
  
  }
  add_action('admin_init', 'restrict_admin', 1);
  
  /**
   * Only use the following action if the role_to_view is NOT -1, i.e. if the default permissions require a logged in user
   */
  if( (get_option('tina_mvc_default_role_to_view') != -1) ) {

    /**
     * Prevent users from seeing the wp-login page unless they have appropriate permissions. This also intercepts the logout command
     */
    function restrict_wp_login() {
        
        if( (strpos( $_SERVER['SCRIPT_FILENAME'], 'wp-login.php') !== FALSE) AND !tina_mvc_user_has_role( explode(',',get_option('tina_mvc_login_logout_redirect')) ) ) {
            
            // if the user wanted to logout we need to intercept that action
            if( !empty($_REQUEST['action']) AND ($_REQUEST['action'] == 'logout') ) {
                check_admin_referer('log-out');
                wp_logout();
            }
            
            wp_redirect( tina_mvc_make_controller_url() );
            exit();
            
        }
    
    }
    add_action('init', 'restrict_wp_login' );

  }
    
}

?>